use std::sync::mpsc::Sender;

use crate::{SortMessage, SortVisualizer};

pub fn sort(vis: &mut SortVisualizer) {
	merge_sort(&vis.tx, &mut vis.arr.clone(), &vis.arr);

	vis.tx.send(SortMessage::Done).unwrap();
}

fn merge_sort(tx: &Sender<SortMessage>, og_arr: &mut [f64], arr: &[f64]) -> Vec<f64> {
	if arr.len() < 2 {
		arr.to_vec()
	}
	else {
		let n = arr.len() / 2;

		let left = merge_sort(tx, og_arr, &arr[0..n]);
		let right = merge_sort(tx, og_arr, &arr[n..]);

		merge(tx, og_arr, &left, &right)
	}
}

fn merge(tx: &Sender<SortMessage>, og_arr: &mut [f64], left: &[f64], right: &[f64]) -> Vec<f64> {
	let mut i = 0;
	let mut j = 0;
	let mut merged: Vec<f64> = vec![];

	while i < left.len() && j < right.len() {
		if left[i] < right[j] {
			merged.push(left[i]);
			i += 1;
		}
		else {
			merged.push(right[j]);
			j += 1;
		}
	}

	if i < left.len() {
		while i < left.len() {
			merged.push(left[i]);
			i += 1;
		}
	}

	if j < right.len() {
		while j < right.len() {
			merged.push(right[j]);
			j += 1;
		}
	}

	for (i, val) in merged.iter().enumerate() {
		og_arr[i] = *val;

		tx.send(SortMessage::Data(og_arr.to_vec().clone())).unwrap();
	}

	merged
}
